#!/usr/bin/env groovy
import com.urbancode.anthill3.*

final def workDir = new File('.').canonicalFile
final def out = System.out

//------------------------------------------------------------------------------
// HELPERS
//------------------------------------------------------------------------------

class BuildInfo {
    def sourcePath
    def sourceLabelName
    def checkoutChangeset
}

/**
 * Method to obtain the checkout source label, changeset and path for a TFS build
 * Output from tftool:
 *   line 1: drop location
 *   line 2: log location
 *   line 3: TFS project name
 *   line 4: sourceLabel@path
 *   line 5: checkout changeset
 *   line 6: comma separated list of changesets for the build
 *
 * @param buildUri
 * @param serverUrl
 * @param tftoolExecutable
 * @return
 */
def BuildInfo getBuildInfo(buildUri, serverUrl, tftoolExecutable) {
    def result = new BuildInfo()
    new CommandHelper(workDir).runCommand("Get TFS Build info for $buildUri",
            [tftoolExecutable, 'buildinfo', "/server:$serverUrl", "/buildUri:$buildUri"]) { proc ->
        proc.consumeProcessErrorStream(out)
        proc.outputStream.close()
        proc.in.eachLine { line, index ->
            println line
            if (index == 3) {
                def labelInfo = line.trim().split('@')
                if (labelInfo.length == 2) {
                    result.sourceLabelName = labelInfo[0]
                    result.sourcePath = labelInfo[1]
                }
                else {
                    println "Found invalid label info line:\n\t$line"
                }
            }
            else if (index == 4) {
                result.checkoutChangeset = line.trim()
            }
        }
    }

    return result
}

//------------------------------------------------------------------------------
// GET ALL INPUT PARAMETERS
//------------------------------------------------------------------------------
final def ahpTool = new AHPTool();

final def props = ahpTool.getStepProperties();

final def String serverUrl = props['integration/serverUrl']
final def String tftoolExecutable = 'tftool' + props['integration/serverVersion'] + '.exe'
final def String startBuildUri = props['startBuildUri']
final def String endBuildUri = props['endBuildUri']
final def String reportName = props['reportName']

try {
    //------------------------------------------------------------------------------
    // CHECK INPUT
    //------------------------------------------------------------------------------
    if (serverUrl == null || serverUrl.trim().length() == 0){
        throw new Exception('TFS Server URL is required but was not found!')
    }

    if (startBuildUri == null || startBuildUri.trim().length() == 0){
        throw new Exception('Start Build URI is required but was not found!')
    }

    if (endBuildUri == null || endBuildUri.trim().length() == 0){
        throw new Exception('End Build URI is required but was not found!')
    }

    if (reportName == null || reportName.trim().length() == 0){
        throw new Exception('Report File Name is required but was not found!')
    }

    //------------------------------------------------------------------------------
    // PREPARE WORKING DIRECTORY
    //------------------------------------------------------------------------------
    workDir.mkdirs()
    if (!workDir.exists()) {
        throw new Exception("Could not create working directory $workDir")
    }
    else if (workDir.isFile()) {
        throw new Exception("Specified working directory is a file: $workDir")
    }

    //------------------------------------------------------------------------------
    // Get build information from TFS
    //------------------------------------------------------------------------------

    def startBuildInfo
    def endBuildInfo
    try {
        startBuildInfo = getBuildInfo(startBuildUri, serverUrl, tftoolExecutable)
        endBuildInfo = getBuildInfo(endBuildUri, serverUrl, tftoolExecutable)
    }
    catch (Exception e) {
        throw new Exception("Error obtaining TFS build information: $e.message")
    }

    // we need to increment the start build checkout changeset to prevent the last changeset from the start build
    // from being included in the changelog report
    def String startChangeset = startBuildInfo.checkoutChangeset
    try {
        startChangeset = 'C' + (Integer.parseInt(startChangeset.substring(1)) + 1)
    }
    catch (NumberFormatException nfe) {
        throw new Exception("Found invalid start build source checkout changeset: $startChangeset")
    }

    new CommandHelper(workDir).runCommand("Create Changelog Report", [
            tftoolExecutable,
            'changelog',
            "/server:$serverUrl", 
            "/v:L$endBuildInfo.sourceLabelName",
            "/from:$startChangeset",
            "/to:$endBuildInfo.checkoutChangeset",
            ">",
            reportName
    ])

}
catch (Throwable t) {
    t.printStackTrace()
    System.exit 1
}